import numpy as np
import torch
import os
import time
import datetime
import pandas as pd
import re
from multiprocessing import Process, cpu_count, Queue
from tweet_collection import mainCollect
from predict import mainPredict
from send_email import send_eamil
import logging
import glob
import psycopg2
import psycopg2.extras as extras
from dotenv import load_dotenv
from preprocess import mainPreProcess

def producer(queue, start_time_iso, end_time_iso, hasgeo):
 # the input start_time_iso and end_time_iso are UTC time in ISO-8601 format
    st=start_time_iso
    start_time_ = datetime.datetime.strptime(start_time_iso, '%Y-%m-%dT%H:%M:%SZ')
    end_time = datetime.datetime.strptime(end_time_iso, '%Y-%m-%dT%H:%M:%SZ')
    time_interval = 60*24   # 24 Hours, in minutes

    while start_time_< end_time:
      # print(start_time_,end_time)
      end_time_ = start_time_ + datetime.timedelta(hours=0,minutes = time_interval)
      end_time_iso_ = datetime.datetime.strftime(end_time_,'%Y-%m-%dT%H:%M:%SZ')
      if end_time_ >= end_time:
          end_time_ = end_time
          end_time_iso_ = end_time_iso
      try:    
      ##########
        processed_tweet_df = mainCollect(start_time_iso, end_time_iso_, hasgeo=hasgeo)
        
        ## send into queue (pass to consumer)
        # queue.put(processed_tweet_df)
        queue.append(processed_tweet_df)


        ##########
      except Exception as e1:
          logging.info("Collection: {}-{} error occurred: {} ".format(start_time_iso, end_time_iso_,e1))
          e1=str(e1)
          email_account.send(to_address,'Data Collection Error',e1) # to_address 'a,b,c',title, content
          break
      if len(queue)%2==0:
        torch.save(queue,'geo_tweets/collected_queue_{}.pt'.format(end_time_))  
        queue =deque()
      
      start_time_ = end_time_
      start_time_iso = end_time_iso_
      
    logging.info('Collection Done')
    email_account.send(to_address,'Data Collection Alert',"Data collection Finished. \n {}-{} ".format(start_time_iso, end_time_iso)) # to_address 'a,b,c',title, content
    print('Done')


#function to insert predictions into the database
#taken from: https://www.geeksforgeeks.org/how-to-insert-a-pandas-dataframe-to-an-existing-postgresql-table/
def toDB(conn, df, table):
  tuples = [tuple(x) for x in df.to_numpy()]
  cols = ','.join(list(df.columns))
  #query to execute
  query = "INSERT INTO %s(%s) VALUES %%s" % (table, cols)
  cursor = conn.cursor()

  try:
    extras.execute_values(cursor, query, tuples)
    conn.commit()
  except (Exception, psycopg2.DatabaseError) as error:
    print("Error: %s" % error)
    conn.rollback()
    cursor.close()
  print("Tweet predictions have succesfully been inserted.")
  cursor.close()

def addTokenColumns(df):
  tokenList = list(df['token_prediction'])
  stringList = list(df['tweet_token'])

  # initialize new columns to add to the dataframe
  symptomCol = []
  locationCol = []
  foodCol = []
  otherCol = []

  # iterate through the tweet tokens and token symbols simultaneously
  for tweet,token in zip(stringList,tokenList):
    # initialize token classifiers per tweet
    symptoms = []
    locations = []
    foods = []
    others = []

    # format token symbols
    letterList = token.split(',')

    # remove all empty strings
    # tweet[:] = [x for x in tweet if x != "''"]

    # iterate through tokens and token symbols, if the token symbol is a symbol we want,
    #      then keep it
    for word,token in zip(tweet,letterList):
      if token == 'B-symptom' or token == 'I-symptom':
        symptoms.append(re.sub(r'\W+','',word).lower())
      elif token == 'B-loc' or token == 'I-loc':
        locations.append(re.sub(r'\W+','',word).lower())
      elif token == 'B-food' or token == 'I-food':
        foods.append(re.sub(r'\W+','',word).lower())
      elif token == 'B-other' or token == 'I-other':
        others.append(re.sub(r'\W+','',word).lower())

    # print(symptoms)

    # Put list in new column list
    symptomCol.append(symptoms)
    locationCol.append(locations)
    foodCol.append(foods)
    otherCol.append(others)

  # declare new columns
  df['symptoms'] = symptomCol
  df['locations'] = locationCol
  df['foods'] = foodCol
  df['others'] = otherCol

  return df

def abbreviate(state):
  us_state_to_abbrev = {
    "Alabama": "AL",
    "Alaska": "AK",
    "Arizona": "AZ",
    "Arkansas": "AR",
    "California": "CA",
    "Colorado": "CO",
    "Connecticut": "CT",
    "Delaware": "DE",
    "Florida": "FL",
    "Georgia": "GA",
    "Hawaii": "HI",
    "Idaho": "ID",
    "Illinois": "IL",
    "Indiana": "IN",
    "Iowa": "IA",
    "Kansas": "KS",
    "Kentucky": "KY",
    "Louisiana": "LA",
    "Maine": "ME",
    "Maryland": "MD",
    "Massachusetts": "MA",
    "Michigan": "MI",
    "Minnesota": "MN",
    "Mississippi": "MS",
    "Missouri": "MO",
    "Montana": "MT",
    "Nebraska": "NE",
    "Nevada": "NV",
    "New Hampshire": "NH",
    "New Jersey": "NJ",
    "New Mexico": "NM",
    "New York": "NY",
    "North Carolina": "NC",
    "North Dakota": "ND",
    "Ohio": "OH",
    "Oklahoma": "OK",
    "Oregon": "OR",
    "Pennsylvania": "PA",
    "Rhode Island": "RI",
    "South Carolina": "SC",
    "South Dakota": "SD",
    "Tennessee": "TN",
    "Texas": "TX",
    "Utah": "UT",
    "Vermont": "VT",
    "Virginia": "VA",
    "Washington": "WA",
    "West Virginia": "WV",
    "Wisconsin": "WI",
    "Wyoming": "WY",
    "District of Columbia": "DC",
    "American Samoa": "AS",
    "Guam": "GU",
    "Northern Mariana Islands": "MP",
    "Puerto Rico": "PR",
    "United States Minor Outlying Islands": "UM",
    "U.S. Virgin Islands": "VI",}
  return us_state_to_abbrev[state]

def getCenterBBOX(coordList):
  longCenter = (coordList[0] + coordList[2])/2
  latCenter = (coordList[1] + coordList[3])/2
  return [longCenter,latCenter]

def formatForDB(df):
  # filter by the positive predicted tweets
  newDF = df[df['sentence_prediction'] == 1]

  # initialize new columns
  city = []
  state = []
  countryCode = []
  username = []
  longitute = []
  latitude = []

  # iterate through rows and update the new columns
  for index, row in newDF.iterrows():
    # add username from user dict, if no user, add blank
    try:
      username.append(row['user']['username'])
    except:
      username.append('')

    # try to decipher the geo dict, if you can't just add blanks
    try:
      # check if place type is city or admin
      if row['geo']['place_type'] == 'city':
        fullName = row['geo']['full_name']
        state.append(fullName[fullName.index(',')+2:])
        city.append(fullName[:fullName.index(',')])
        countryCode.append(row['geo']['country_code'])
        coordinate = getCenterBBOX(row['geo']['geo']['bbox'])
        longitute.append(coordinate[0])
        latitude.append(coordinate[1])
      elif row['geo']['place_type'] == 'admin':
        fullName = row['geo']['full_name']
        state.append(abbreviate(fullName[:fullName.index(',')]))
        city.append('')
        countryCode.append(row['geo']['country_code'])
        longitute.append(None)
        latitude.append(None)
      # was not city nor admin
      else:
        city.append('')
        state.append('')
        countryCode.append('')
        longitute.append(None)
        latitude.append(None)

    except:
      city.append('')
      state.append('')
      countryCode.append('')
      longitute.append(None)
      latitude.append(None)

  # insert new columns
  newDF.insert(4,'username',username)
  newDF.insert(4,'state',state)
  newDF.insert(4,'city',city)
  newDF.insert(4,'countryCode',countryCode)
  newDF['longitude'] = longitute
  newDF['latitude'] = latitude

  # filter by US in country code
  newDF = newDF[newDF['countryCode'] == 'US']
    
  # drop un-needed columns
  newDF = newDF.drop(columns = ['author_id', 'geo','countryCode', 'user', 'tweet_token', 'token_prediction', 'sentence_prediction'])

  return newDF

def getNextDataIndex():
  dataFiles = []
  for f in os.listdir('raw_data'):
    # check only text files
    if f.endswith('.pt'):
        dataFiles.append(int(f[:-3]))
  if len(dataFiles) == 0:
    return 0
  return max(dataFiles) + 1

def consumer(queue,filename):
  ## Step 1: prediction
  i=1
  while True:
    
    if len(queue)==0:
      break
    df = queue.popleft()
  
    df_with_pred = mainPredict(df)
    df_with_pred = mainPreProcess(df_with_pred)
    logging.info("{}-batch_{} Prediction Finished".format(filename,i))    
    i+=1

 # email_account.send(to_address,'Data Collection Alert',"{} Prediction Finished".format(filename)) # to_address 'a,b,c',title, content

  ##place all of predictions into queue (or csv) to look at raw data later
  #predqueue.append(df_with_pred)
  #torch.save(queue,'prediction_queues/pred_queue_{}'.format(end_time_)) #not sure what to do here? 
  
    ######  TO DO, Step 2 ########
    # send "df_with_pred" to database
    load_dotenv()
    conn = psycopg2.connect(
      host = os.environ.get('DB_HOST'),
      database = os.environ.get('DB_NAME'),
      user = os.environ.get('DB_USER'),
      password = os.environ.get('DB_PASSWORD'))

    #in here, need to first put in function that will create food, symptom, location columns
    df_with_pred = addTokenColumns(df_with_pred)
   
    #save raw data before filtering data
    torch.save(df_with_pred, 'raw_data/' + str(getNextDataIndex()) + '.pt')

    #alter table for database
    df_with_pred1 = formatForDB(df_with_pred)
    toDB(conn,df_with_pred1,'final_tweets')

if __name__ == '__main__':
    # logging setting
    from collections import deque
    queue = deque()
    for handler in logging.root.handlers[:]:
      logging.root.removeHandler(handler)# Create and configure logger
    logging.basicConfig(filename="logs/Prediction.log",level=logging.INFO,format="%(asctime)s %(message)s")
    
    # send email alert setting
    global email_account
    global to_address
      
#email_account = send_eamil("wpi_uiuc_fs@163.com" ,"HTKSJYOPLAFPQEZL" ) # Don't change this: host_account,password
   # to_address = 'rhu@wpi.edu' #gr-food-safety-swag@wpi.edu,

    #####
    data_dir = './geo_tweets_test'
    # list to store files
    file_list = []

    # Keep track of runtime
    startRuntime = time.time()

    # Iterate directory
    for f in os.listdir(data_dir):
        # check only text files
        if f.endswith('.pt'):
            file_list.append(f)
    for filename in file_list:
      file_path = os.path.join(data_dir, filename)
      queue =torch.load(file_path)
      consumer(queue,filename)

    print('-------------------------------------------')
    print('Prediction Runtime: ', (time.time() - startRuntime)/3600, 'hours')
